/*
    Proteus RPG Engine - A flexible 3D story based RPG engine.
    Copyright (C) 2002  Sam Yatchmenoff

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <windows.h>

#include <GL/gl.h>
//#include <GL/glut.h>
#include <iostream>
#include <sstream>
#include <string>
#include "model.h"
#include "tinyxml.h"

using namespace std;


void Model::set_location(float x, float y, float z) {
  loc.x = x;
  loc.y = y;
  loc.z = z;
}

void Model::set_rotation(float x, float y, float z) {
  rot.x = x;
  rot.y = y;
  rot.z = z;
}

Model::Model() {
   selected = 0;
   r = 1.0f;
   g = 1.0f;
   b = 1.0f;
}

Model::~Model() {
}

void Model::color(GLfloat mr, GLfloat mg, GLfloat mb)
{
   r = mr;
   g = mg;
   b = mb;
}

void Model::render_mesh(int m) {
	glBegin(GL_TRIANGLES);
	for(int i = 0; i < meshes[m].num_triangles; i++) {
		glVertex3f(vertices[triangles[meshes[m].tris[i]].a].x,
					  vertices[triangles[meshes[m].tris[i]].a].y,
					  vertices[triangles[meshes[m].tris[i]].a].z);
		glVertex3f(vertices[triangles[meshes[m].tris[i]].b].x,
					  vertices[triangles[meshes[m].tris[i]].b].y,
					  vertices[triangles[meshes[m].tris[i]].b].z);
		glVertex3f(vertices[triangles[meshes[m].tris[i]].c].x,
					  vertices[triangles[meshes[m].tris[i]].c].y,
					  vertices[triangles[meshes[m].tris[i]].c].z);
	}
	glEnd();
}

void Model::render_skeleton(Node *node) {
   render_mesh(node->mesh);
		
	for(int i = 0; i < node->num_children; i++) {
   	render_skeleton(node->children[i]);
	}
}

	
void Model::render() {
   glPushMatrix();

   glTranslatef(loc.x, loc.y, loc.z);
   glRotatef(rot.x, 1.0, 0.0, 0.0);
   glRotatef(rot.y, 0.0, 1.0, 0.0);
   glRotatef(rot.z, 0.0, 0.0, 1.0);

	if(root != NULL) {
		render_skeleton(root);
	} else {
		for(int i = 0; i < num_meshes; i++) {
			render_mesh(i);
		}
	}
   
	glPopMatrix();
}

int Model::load(const string &filename) {
   TiXmlDocument file(filename);
   if(!(file.LoadFile())) {
      cerr << "Unable to load model '" << filename << "'\n";
		//return -1;
   }

	TiXmlNode *top;
	assert(top = file.FirstChild("ProteusModel"));

   TiXmlNode *node;

	// Load skeleton
   node = top->FirstChild("Skeleton");
	if(node == NULL) {
		root = NULL;
	} else {
	   root = new Node;
		istringstream skelss(node->FirstChild()->Value());
		load_skeleton(skelss, root);  // run recursive skeleton loader
	}

	// Load vertices
	assert(node = top->FirstChild("Vertices"));
   istringstream vcss(*(node->ToElement()->Attribute("num")));
   vcss >> num_vertices;
   vertices = new Vertex[num_vertices];
   istringstream vertexss(node->FirstChild()->Value());
   for(int i = 0; i < num_vertices; i++) {
      vertexss >> vertices[i].x >> vertices[i].y >> vertices[i].z;

		if(root != NULL) {
			vertexss >> vertices[i].bone;
		}
   }

	// Load triangles
   assert(node = node->NextSibling("Triangles"));
   istringstream tcss(*(node->ToElement()->Attribute("num")));
   tcss >> num_triangles;
   triangles = new Triangle[num_triangles];
   istringstream triangless(node->FirstChild()->Value());
   for(int i = 0; i < num_triangles; i++) {
      triangless >> triangles[i].a >> triangles[i].b >> triangles[i].c;
   }

	// Load meshes
	assert(node = node->NextSibling("Meshes"));
	istringstream mcss(*(node->ToElement()->Attribute("num")));
	mcss >> num_meshes;
	meshes = new Mesh[num_meshes];
	TiXmlNode *mesh = node->FirstChild("Mesh");
	for(int i = 0; i < num_meshes; i++) {
		istringstream imcss(*(mesh->ToElement()->Attribute("num_triangles")));
		imcss >> meshes[i].num_triangles;
		meshes[i].tris = new int[meshes[i].num_triangles];
		istringstream imeshss(mesh->FirstChild()->Value());
		for(int n = 0; n < meshes[i].num_triangles; n++) {
			imeshss >> meshes[i].tris[n];	
		}
	}

	return 0;
}

void Model::load_skeleton(istringstream &ss, Node *node) {
	ss >> node->x >> node->y >> node->z >> node->mesh >> node->num_children;

	node->children = new (Node*)[node->num_children];
	for(int i = 0; i < node->num_children; i++) {
		node->children[i] = new Node;
		load_skeleton(ss, node->children[i]);
	}
}
