/*
    Proteus RPG Engine - A flexible 3D story based RPG engine.
    Copyright (C) 2002  Sam Yatchmenoff

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <windows.h>

#include <iostream>
#include <string>
#include <sstream>
#include "main.h"
#include "object.h"
#include "tinyxml.h"

using namespace std;

Object::Object() { }

int Object::load(const string &filename) {
#ifdef DEBUG
	cout << "Loading " << filename << '\n';
#endif
 
	TiXmlDocument objectcfg;
	TiXmlDocument typecfg;
	TiXmlNode *node;
	TiXmlText *text;

	objectcfg.LoadFile(filename);
  
	// Some error checking
	if(objectcfg.Error()) {
		cerr << filename << " could not be loaded\n";
		return -1;
	}
	if(!(node = objectcfg.FirstChild("Object"))) {
		cerr << filename << " is not a valid object file\n";
		return -1;
	}

	// What type of object is this?
	node = node->FirstChild("Type");
	type = node->FirstChild()->ToText()->Value();
#ifdef DEBUG
	cout << "Type: " << type << endl;
#endif

	// Grab the model
	model = Resource<Model>(get_module_name() + "/data/models/" + type + ".pmd", Level);
	string typecfg_name = get_module_name() + "/types/objects/" + type + ".xml";

	if(!typecfg.LoadFile(typecfg_name)) {
		cerr << "Invalid type!  Oh no!\n";
		return -1;
	}
	assert(node = typecfg.FirstChild("ObjectType"));
	assert(node = node->FirstChild("Actions"));
	for(TiXmlElement *action = node->FirstChildElement(); action; action = action->NextSiblingElement()) {
		Action tmpaction;
		tmpaction.type = *(action->Attribute("type"));
		tmpaction.target = *(action->Attribute("target"));
		for(TiXmlElement *effect = action->FirstChildElement(); effect; effect = effect->NextSiblingElement()) {
			Effect tmpeffect;
			tmpeffect.type = *(effect->Attribute("type"));
			tmpeffect.subtype = *(effect->Attribute("subtype"));
			tmpeffect.targettype = *(effect->Attribute("targettype"));
			tmpeffect.targetname = *(effect->Attribute("targetname"));
			tmpeffect.value = effect->FirstChild()->Value();
			tmpaction.effects.push_back(tmpeffect);
			
		}
		actions.push_back(tmpaction);
	}

	// Get the location of the object
	node = objectcfg.FirstChild("Object")->FirstChild("Location");
	node = node->FirstChild("X");
	string loc;
	loc = node->FirstChild()->Value();
	node = node->NextSibling("Y");
	loc += ' '; loc += node->FirstChild()->ToText()->Value();
	node = node->NextSibling("Z");
	loc += ' '; loc += node->FirstChild()->Value();
	istringstream sstr(loc);
	sstr >> location.x >> location.y >> location.z;
}

void Object::use_on(Character *target) {
	list<Action>::const_iterator i;
	for(i = actions.begin(); i != actions.end(); i++) {
		if(i->target == "character" && i->type == "use") {
			list<Effect>::const_iterator e;
			for(e = i->effects.begin(); e != i->effects.end(); e++) {
				if(e->type == "modify") {
					if(e->subtype == "add") {
						if(e->targettype == "stat") {
							if(e->targetname == "health") {
								istringstream value(e->value);
								int op;
								value >> op;
								target->get_status()->HP += op;
							}
						}
					}
				}
			}
			break;
		}
	}
}

/*
// For debugging
ostream &operator<<(ostream &s, const Object &o) {
	map<string,int> *attribs = o.get_attributes();
	map<string,int>::const_iterator iterator;
	for(map<string,int>::const_iterator iterator = attribs->begin();
		 iterator != attribs->end(); iterator++) {
		cout << iterator->first << '\t' << iterator->second << '\n';
	}
}
*/
